#!/usr/bin/perl
#
# Program that extract various graphs from data set.
#

# Function that prints usage and exits
sub usage {
	print "\nUsage: $0 dimacs_file\n\n";
	exit;
}

# Handle input files
$num_args = $#ARGV + 1;
if ($num_args != 1) {
	usage();
}
$file = $ARGV[0];
open(IN, $file) or die "Couldn't read $file: $!";
@lines = <IN>;
close(IN);

# Store ids
$num_items;
$num_users;
$k;
$num_arcs = 0;
@arcs;
@users_keep;	# ids of the users to keep
@users_map;		# keep track of user substitution


# Read the file
$first = true;
foreach $line (@lines) {
	
	if ($first && $line =~ /(\d+)\s(\d+)\s(\d+)/) {	# First line
		
		$num_items	= $1;
		$num_users	= $2;
		$k 			= $3;
		$first = false;
		
		for (my $i = 1; $i <= $num_users; $i++) {
			$users_keep[$i] 	= 0;
			$users_map[$i] 	= 0;
		}
	
	} else {			# Other lines (arcs)
		$arcs[$num_arcs] = [ split ' ', $line ];
		# Re-normalize the users
		$arcs[$num_arcs][1] -= $num_items;
		$users_keep[$arcs[$num_arcs][1]] = 1;
		#print "$arcs[$num_arcs][0] $arcs[$num_arcs][1]\n";
		$num_arcs++;
	}

}

# Get the new mappings of the users
$temp = $num_users;
for (my $i = 1; $i <= $temp; $i++) {
	if ($users_keep[$i] == 0) {
		$num_users--;
		for (my $j = $i + 1; $j <= $temp; $j++) {
			$users_map[$j]--;
		}
	}
}

# Output the final graph
print "$num_items $num_users $k\n";

# Output the arcs
for(my $i = 0; $i < $num_arcs; $i++) {
	
	$new_user = $arcs[$i][1] + $users_map[$arcs[$i][1]] + $num_items;
	print "$arcs[$i][0] $new_user\n";
}

